Build Packages / Create release (push) Successful in 23s
Build Packages / build:rugnux-tgz (x86_64) (push) Successful in 10m6s
Build Packages / build:rugnux:aarch64 (cross) (push) Successful in 9m6s
Build Packages / build:viewer-tgz:cpu (push) Successful in 11m15s
Build Packages / build:viewer-tgz:cuda (push) Successful in 12m21s
Build Packages / build:windows:nocuda (push) Successful in 17m9s
Build Packages / build:windows:cuda (push) Successful in 19m49s
Build Packages / HDF5 consumer tests (DIALS, XDS) (push) Successful in 25m42s
Build Packages / build:rpm (ubuntu2204_nocuda) (push) Successful in 16m0s
Build Packages / build:rpm (ubuntu2404_nocuda) (push) Successful in 14m54s
Build Packages / build:rpm (rocky8_nocuda) (push) Successful in 17m7s
Build Packages / build:rugnux:windows (push) Successful in 10m47s
Build Packages / build:rpm (rocky9_nocuda) (push) Successful in 17m4s
Build Packages / build:rpm (rocky8_sls9) (push) Successful in 17m8s
Build Packages / Generate python client (push) Successful in 45s
Build Packages / Build documentation (push) Successful in 1m45s
Build Packages / build:rpm (rocky9_sls9) (push) Successful in 19m23s
Build Packages / build:rpm (ubuntu2204) (push) Successful in 19m20s
Build Packages / build:rpm (ubuntu2404) (push) Successful in 18m43s
Build Packages / build:rpm (rocky8) (push) Successful in 19m31s
Build Packages / build:rpm (rocky9) (push) Successful in 20m16s
Build Packages / Unit tests (push) Successful in 1h41m19s
* Fixed a `jfjoch_broker` crash during indexing: sorting no longer misbehaves on non-finite values, and GPU FFT indexer kernel launches are now error-checked. * rugnux needs about a third less peak memory to scale, merge and post-refine rotation data, with identical results. * `rugnux --model`: the placed coordinate file carries the space group its own coordinates obey, and says so when that is not the group the reflection files beside it carry. * `jfjoch_viewer`: fixes in the dataset plots, inspector and layout; spot markers lose their black outline by default (a checkbox under "Image features" restores it) and the highest-pixel markers are white boxes around the pixel. Reviewed-on: #80 Co-authored-by: Filip Leonarski <filip.leonarski@psi.ch>
150 lines
5.6 KiB
Plaintext
150 lines
5.6 KiB
Plaintext
// SPDX-FileCopyrightText: 2025 Filip Leonarski, Paul Scherrer Institute <filip.leonarski@psi.ch>
|
|
// SPDX-License-Identifier: GPL-3.0-only
|
|
|
|
#include "AzIntEngineGPU.h"
|
|
|
|
inline void cuda_err(cudaError_t val) {
|
|
if (val != cudaSuccess)
|
|
throw JFJochException(JFJochExceptionCategory::GPUCUDAError, cudaGetErrorString(val));
|
|
}
|
|
|
|
__global__
|
|
void gpu_azim_shared(
|
|
const uint16_t *__restrict__ pixel_to_bin,
|
|
const float *__restrict__ corrections,
|
|
const int32_t *__restrict__ input_buffer,
|
|
float *__restrict__ azint_sum,
|
|
float *__restrict__ azint_sum2,
|
|
uint32_t *__restrict__ azint_count,
|
|
size_t num_pixels,
|
|
int azint_bins) {
|
|
extern __shared__ float shared[];
|
|
|
|
float *s_sum = shared;
|
|
float *s_sum2 = &s_sum[azint_bins];
|
|
uint32_t *s_count = (uint32_t *) &s_sum2[azint_bins];
|
|
|
|
// Initialize shared memory
|
|
for (int i = threadIdx.x; i < azint_bins; i += blockDim.x) {
|
|
s_sum[i] = 0.0f;
|
|
s_sum2[i] = 0.0f;
|
|
s_count[i] = 0;
|
|
}
|
|
|
|
__syncthreads();
|
|
|
|
for (size_t idx = blockIdx.x * blockDim.x + threadIdx.x;
|
|
idx < num_pixels;
|
|
idx += blockDim.x * gridDim.x) {
|
|
uint16_t bin = pixel_to_bin[idx];
|
|
|
|
int32_t v = input_buffer[idx];
|
|
bool valid = (v != INT32_MIN) & (v != INT32_MAX);
|
|
|
|
if (bin < azint_bins && valid) {
|
|
const float val = static_cast<float>(v) * corrections[idx];
|
|
const float val2 = val * val;
|
|
atomicAdd(&s_sum[bin], val);
|
|
atomicAdd(&s_sum2[bin], val2);
|
|
atomicAdd(&s_count[bin], 1);
|
|
}
|
|
}
|
|
|
|
__syncthreads();
|
|
|
|
// Merge to global memory
|
|
for (unsigned int i = threadIdx.x; i < azint_bins; i += blockDim.x) {
|
|
atomicAdd(&azint_sum[i], s_sum[i]);
|
|
atomicAdd(&azint_sum2[i], s_sum2[i]);
|
|
atomicAdd(&azint_count[i], s_count[i]);
|
|
}
|
|
}
|
|
|
|
__global__
|
|
void gpu_azim(
|
|
const uint16_t *__restrict__ pixel_to_bin,
|
|
const float *__restrict__ corrections,
|
|
const int32_t *__restrict__ input_buffer,
|
|
float *__restrict__ azint_sum,
|
|
float *__restrict__ azint_sum2,
|
|
uint32_t *__restrict__ azint_count,
|
|
size_t num_pixels,
|
|
int azint_bins) {
|
|
for (size_t idx = blockIdx.x * blockDim.x + threadIdx.x;
|
|
idx < num_pixels;
|
|
idx += blockDim.x * gridDim.x) {
|
|
uint16_t bin = pixel_to_bin[idx];
|
|
|
|
int32_t v = input_buffer[idx];
|
|
bool valid = (v != INT32_MIN) & (v != INT32_MAX);
|
|
|
|
if (bin < azint_bins && valid) {
|
|
const float val = static_cast<float>(v) * corrections[idx];
|
|
const float val2 = val * val;
|
|
atomicAdd(&azint_sum[bin], val);
|
|
atomicAdd(&azint_sum2[bin], val2);
|
|
atomicAdd(&azint_count[bin], 1);
|
|
}
|
|
}
|
|
}
|
|
|
|
AzIntEngineGPU::AzIntEngineGPU(const AzimuthalIntegrationMapping &integration, std::shared_ptr<CudaStream> stream)
|
|
: AzIntEngine(integration),
|
|
stream(stream),
|
|
gpu_sum(azint_bins),
|
|
gpu_sum2(azint_bins),
|
|
gpu_count(azint_bins),
|
|
cpu_sum_reg(azint_sum),
|
|
cpu_sum2_reg(azint_sum2),
|
|
cpu_count_reg(azint_count) {
|
|
|
|
int device = 0;
|
|
cuda_err(cudaGetDevice(&device)); // this worker's GPU, not necessarily 0
|
|
cudaDeviceProp prop{};
|
|
cuda_err(cudaGetDeviceProperties(&prop, device));
|
|
|
|
threads = 128;
|
|
blocks = 4 * prop.multiProcessorCount;
|
|
shared_size = prop.sharedMemPerBlock;
|
|
shared_needed = azint_bins * (2 * sizeof(float) + sizeof(uint32_t));
|
|
|
|
// Geometry-only, so shared per GPU: the first engine on this device uploads them, the rest reuse
|
|
// them. Keyed by the mapping's own vectors, which outlive every engine built from it.
|
|
gpu_azint_correction = SharedDeviceTable(integration.Corrections().data(), npixel,
|
|
integration.Corrections().data(),
|
|
integration.GetCorrectionsChecksum(), *stream);
|
|
gpu_pixel_to_bin = SharedDeviceTable(integration.GetPixelToBin().data(), npixel,
|
|
integration.GetPixelToBin().data(),
|
|
integration.GetPixelToBinChecksum(), *stream);
|
|
}
|
|
|
|
void AzIntEngineGPU::Run(const ImagePreprocessorBuffer &image, AzimuthalIntegrationProfile &profile) {
|
|
if (image.size() != integration.GetPixelToBin().size())
|
|
throw std::runtime_error("ImageSpotFinder::AzimIntegration: Mismatch in size");
|
|
cuda_err(cudaMemsetAsync(gpu_sum, 0, sizeof(float) * azint_bins, *stream));
|
|
cuda_err(cudaMemsetAsync(gpu_sum2, 0, sizeof(float) * azint_bins, *stream));
|
|
cuda_err(cudaMemsetAsync(gpu_count, 0, sizeof(uint32_t) * azint_bins, *stream));
|
|
|
|
if (shared_needed < shared_size) {
|
|
gpu_azim_shared<<<blocks, threads, shared_needed, *stream>>>(
|
|
gpu_pixel_to_bin->get(),gpu_azint_correction->get(),image.getGPUBuffer(), gpu_sum, gpu_sum2,
|
|
gpu_count, npixel, azint_bins
|
|
);
|
|
cuda_err(cudaGetLastError());
|
|
} else {
|
|
gpu_azim<<<blocks, threads, 0, *stream>>>(
|
|
gpu_pixel_to_bin->get(),gpu_azint_correction->get(),image.getGPUBuffer(), gpu_sum, gpu_sum2,
|
|
gpu_count, npixel, azint_bins
|
|
);
|
|
cuda_err(cudaGetLastError());
|
|
}
|
|
|
|
cudaMemcpyAsync(azint_sum.data(), gpu_sum, sizeof(float) * azint_bins, cudaMemcpyDeviceToHost, *stream);
|
|
cudaMemcpyAsync(azint_sum2.data(), gpu_sum2, sizeof(float) * azint_bins, cudaMemcpyDeviceToHost, *stream);
|
|
cudaMemcpyAsync(azint_count.data(), gpu_count, sizeof(uint32_t) * azint_bins, cudaMemcpyDeviceToHost, *stream);
|
|
cuda_err(cudaStreamSynchronize(*stream));
|
|
|
|
profile.Clear(integration);
|
|
profile.Add(azint_sum, azint_sum2, azint_count);
|
|
}
|